#include "func_math.h"

void calc_azi_elev(const double gmst_hour, const double obj_RA_Dec[], const double long_lat[], vector<double> &azi_elev)
{
  const double math_pi = 3.14159265358979323;
  const double degree_to_radian = math_pi / 180;
  const double radian_to_degree = 180 / math_pi;
  azi_elev.resize(2);

  const double hour_angle = gmst_hour + long_lat[0] - obj_RA_Dec[0];
  const double hour_angle_rad = hour_angle * 15 * degree_to_radian;

  const double obj_Dec_rad = obj_RA_Dec[1] * degree_to_radian;
  const double latitude_rad = long_lat[1] * degree_to_radian;

  const double cosHour_sinAzi = -cos(obj_Dec_rad) * sin(hour_angle_rad);
  const double sinHour = sin(obj_Dec_rad) * sin(latitude_rad) + cos(obj_Dec_rad) * cos(hour_angle_rad) * cos(latitude_rad);
  const double cosHour_cosAzi = sin(obj_Dec_rad) * cos(latitude_rad) - cos(obj_Dec_rad) * cos(hour_angle_rad) * sin(latitude_rad);

  azi_elev[1] = asin(sinHour) * radian_to_degree;
  azi_elev[0] = atan2(cosHour_sinAzi, cosHour_cosAzi) * radian_to_degree;
  if(azi_elev[0] < 0) azi_elev[0] += 360;
}

double func_center(vector<double> data, int naxis1, int naxis2, int init_x, int init_y, int range, int axis)
{
  int count1, count2, range2 = int((range + 1) / 2);
  double sum_flux = 0, sum_center = 0, tmp;
  
  for(count1 = init_x - range2 + 1; count1 < init_x + range2; count1++) {
    for(count2 = init_y - range2 + 1; count2 < init_y + range2; count2++) {
      tmp = count1 + count2 * naxis1; 

      if(tmp >= 0 && tmp < naxis1 * naxis2) {
        sum_flux += func_square(data[tmp]);
	if(axis == 1) sum_center += func_square(data[tmp]) * count1;
	else if (axis == 2) sum_center += func_square(data[tmp]) * count2;
      }

    }
  }

//cout << "sum_flux = " << sum_flux << "\n";
  if(sum_flux == 0 && axis == 1) return init_x;
  else if(sum_flux == 0 && axis == 2) return init_y;
  else return sum_center / sum_flux;
}

double func_box_flux(vector<double> data, int naxis1, int naxis2, double center_x, double center_y, int range)
{
  int count1, count2, range2 = int((range + 1) / 2);
  double sum_flux = 0, fraction_x, fraction_y, fraction_xy, tmp;
  
  for(count1 = (int)center_x - range2; count1 < (int)center_x + range2; count1++) {
    for(count2 = (int)center_y - range2; count2 < (int)center_y + range2; count2++) {
      tmp = count1 + count2 * naxis1; 

      fraction_x = (double)range / 2 + 1 - abs(center_x - (double)count1);
      fraction_y = (double)range / 2 + 1 - abs(center_y - (double)count2);
      if (fraction_x > 1) fraction_x = 1;
      if (fraction_x < 0) fraction_x = 0;
      if (fraction_y > 1) fraction_y = 1;
      if (fraction_y < 0) fraction_y = 0;
      fraction_xy = fraction_x * fraction_y;

//cout << "fraction_xy = " << fraction_xy << "\n";

      if(tmp >= 0 && tmp < naxis1 * naxis2) {
        sum_flux += data[tmp] * fraction_xy;
      }

    }
  }

//cout << "sum_flux = " << sum_flux << "\n";

  return sum_flux;
}

double func_rotate_x(double input_x, double input_y, double center_x, double center_y, double rotate){
  return center_x + (input_x - center_x) * cos(rotate) - (input_y - center_y) * sin(rotate);
}

double func_rotate_y(double input_x, double input_y, double center_x, double center_y, double rotate){
  return center_y + (input_x - center_x) * sin(rotate) + (input_y - center_y) * cos(rotate);
}

void func_v_sigma(const vector<double> &v_input, vector<double> &v_output){
  int count1, size;
  double mean = 0, sigma = 0, d_size;

  size = (int)v_input.size();
  d_size = (double)size;

  for (count1 = 0; count1 < size; count1++) {
    mean += v_input[count1];
  }
  mean /= d_size;

  for (count1 = 0; count1 < size; count1++) {
    sigma += func_square(v_input[count1] - mean);
  }
  sigma = sqrt(sigma / d_size);

  if (sigma == 0) {
    for (count1 = 0; count1 < size; count1++) {
      v_output[count1] = 0;
    }
  }
  else {
    for (count1 = 0; count1 < size; count1++) {
      v_output[count1] = (v_input[count1] - mean) / sigma;
    }
  }

/*
  cout << "mean = " << mean << " sigma = " << sqrt(sigma) << "\n";
  for (count1 = 0; count1 < size; count1++) {
    cout << "input =";
    for (count1 = 0; count1 < size; count1++) {
      cout << " " << v_input[count1];
    }
    cout << "\n";
  }

  for (count1 = 0; count1 < size; count1++) {
    cout << "sigma =";
    for (count1 = 0; count1 < size; count1++) {
      cout << " " << v_output[count1];
    }
    cout << "\n";
  }
*/
}

double func_median(vector<double> data)
{
  double median;
  size_t size = data.size();

  sort(data.begin(), data.end());

  if (size % 2 == 0) median = (data[size / 2 - 1] + data[size / 2]) / 2;
  else median = data[size / 2];

//cout << median << "\n";
  return median;
}

double func_mean(vector<double> data)
{
  unsigned int count;
  double mean = 0;

  for (count = 0; count < data.size(); count++)
    mean += data[count];

  return mean / data.size();
}

double func_stddev(vector<double> data)
{
  unsigned int count;
  double mean = 0, variance = 0;

  for (count = 0; count < data.size(); count++)
    mean += data[count];

  mean /= data.size();

  for (count = 0; count < data.size(); count++)
    variance += func_square(data[count] - mean);

  return sqrt(variance / data.size());
}

double func_linear_approx(double x1, double x2, double y1, double y2, double x)
{
  if (x1 == x2) return y1;
  else return (y2 - y1) / (x2 - x1) * (x - x1) + y1;
}

double func_square(double x)
{
  return x * x;
}
